@@ -13,6 +13,8 @@ import subprocess
 import sys
 import os
 import itertools
+import collections
+from functools import reduce
 
 class ListOpError(Exception):
     pass
@@ -55,13 +57,13 @@ class grep(TextOp):
         ('info', '1'), ('warning', '2'), ('info', '2')]
         >>> input | cutca(r'(\D+)(\d+)').grep('1',1).tolist()
         [('error', '1'), ('warning', '1'), ('info', '1')]
-        >>> input | cutdct(r'(?P<level>\D+)(?P<nb>\d+)') #doctest: +NORMALIZE_WHITESPACE
-        [{'nb': '1', 'level': 'error'}, {'nb': '2', 'level': 'error'},
-        {'nb': '1', 'level': 'warning'}, {'nb': '1', 'level': 'info'},
-        {'nb': '2', 'level': 'warning'}, {'nb': '2', 'level': 'info'}]
-        >>> input | cutdct(r'(?P<level>\D+)(?P<nb>\d+)').grep('1','nb').tolist() #doctest: +NORMALIZE_WHITESPACE
-        [{'nb': '1', 'level': 'error'}, {'nb': '1', 'level': 'warning'},
-        {'nb': '1', 'level': 'info'}]
+        >>> input | cutdct(r'(?P<level>\D+)(?P<nb>\d+)')  #doctest: +NORMALIZE_WHITESPACE
+        [{'level': 'error', 'nb': '1'}, {'level': 'error', 'nb': '2'},
+        {'level': 'warning', 'nb': '1'}, {'level': 'info', 'nb': '1'},
+        {'level': 'warning', 'nb': '2'}, {'level': 'info', 'nb': '2'}]
+        >>> input | cutdct(r'(?P<level>\D+)(?P<nb>\d+)').grep('1','nb').tolist()  #doctest: +NORMALIZE_WHITESPACE
+        [{'level': 'error', 'nb': '1'}, {'level': 'warning', 'nb': '1'},
+        {'level': 'info', 'nb': '1'}]
         >>> [{'more simple':1},{'way to grep':2},{'list of dicts':3}] | grep('way').tolist()
         [{'way to grep': 2}]
         >>> [{'more simple':1},{'way to grep':2},{'list of dicts':3}] | grep('3').tolist()
@@ -75,10 +77,10 @@ class grep(TextOp):
     def op(cls,text,pattern=None, key=None, has_key=None, attr=None, has_attr=None, *args,**kwargs):
         if pattern is None:
             pattern = cls.pattern
-        regex = re.compile(pattern,cls.flags) if isinstance(pattern,basestring) else pattern
+        regex = re.compile(pattern,cls.flags) if isinstance(pattern,str) else pattern
         for line in cls._tolist(text):
             try:
-                if isinstance(line,basestring):
+                if isinstance(line,str):
                     if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
                         yield line
                 elif has_key is not None:
@@ -204,11 +206,11 @@ class grepc(TextOp):
             return 0
         if pattern is None:
             pattern = cls.pattern
-        regex = re.compile(pattern,cls.flags) if isinstance(pattern,basestring) else pattern
+        regex = re.compile(pattern,cls.flags) if isinstance(pattern,str) else pattern
         count = 0
         for line in cls._tolist(text):
             try:
-                if isinstance(line,basestring):
+                if isinstance(line,str):
                     if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
                         count += 1
                         if cls.exit_on_found:
@@ -415,7 +417,7 @@ class wcount(TextOp):
         count = 0
         for line in cls._tolist(text):
             try:
-                if isinstance(line,basestring):
+                if isinstance(line,str):
                     words = re.split('\s+',line.strip())
                 elif isinstance(line,list):
                     if key is None:
@@ -424,7 +426,7 @@ class wcount(TextOp):
                         words = re.split('\s+',stru(line[key]).strip())
                 elif isinstance(line,dict):
                     if key is None:
-                        words = line.values()
+                        words = list(line.values())
                     else:
                         words = re.split('\s+',stru(line[key]).strip())
                 else:
@@ -511,16 +513,16 @@ class doformat(TextOp):
         str: formatted input
 
     Examples:
-        >>> print ['Eric','Guido'] | doformat('First name : {0}','\n')
+        >>> print(['Eric','Guido'] | doformat('First name : {0}','\n'))
         First name : Eric
         First name : Guido
         >>> ['Eric','Guido'] | doformat('{0} <{0}@github.com>',',')
         'Eric <Eric@github.com>,Guido <Guido@github.com>'
         >>> ctx = {'hostname' : 'pcubuntu'}
-        >>> print ['eth0','eth1'] | doformat('{hostname}/network/{0}','\n',context=ctx)
+        >>> print(['eth0','eth1'] | doformat('{hostname}/network/{0}','\n',context=ctx))
         pcubuntu/network/eth0
         pcubuntu/network/eth1
-        >>> print ['eth0','eth1'] | doformat('{nodename}/network/{4}','\n',ctx,'(unknown)')
+        >>> print(['eth0','eth1'] | doformat('{nodename}/network/{4}','\n',ctx,'(unknown)'))
         (unknown)/network/(unknown)
         (unknown)/network/(unknown)
     """
@@ -553,10 +555,10 @@ class formatitems(TextOp):
         'key1:val1, key2:val2'
         >>> ctx = {'hostname' : 'pcubuntu'}
         >>> d = [['Dimm1','1024'],['Dimm2','512']]
-        >>> print d | formatlists('{hostname}/{0} : {1} MB','\n',ctx)
+        >>> print(d | formatlists('{hostname}/{0} : {1} MB','\n',ctx))
         pcubuntu/Dimm1 : 1024 MB
         pcubuntu/Dimm2 : 512 MB
-        >>> print d | formatlists('{nodename}/{0} : {4} MB','\n',ctx,'??')
+        >>> print(d | formatlists('{nodename}/{0} : {4} MB','\n',ctx,'??'))
         ??/Dimm1 : ?? MB
         ??/Dimm2 : ?? MB
     """
@@ -589,10 +591,10 @@ class formatlists(TextOp):
         'key1:val1 (help1), key2:val2 (help2)'
         >>> ctx = {'hostname' : 'pcubuntu'}
         >>> d = [['Dimm1','1','GB'],['Dimm2','512','MB']]
-        >>> print d | formatlists('{hostname}/{0} : {1} {2}','\n',ctx)
+        >>> print(d | formatlists('{hostname}/{0} : {1} {2}','\n',ctx))
         pcubuntu/Dimm1 : 1 GB
         pcubuntu/Dimm2 : 512 MB
-        >>> print d | formatlists('{nodename}/{0} : {1} {4}','\n',ctx,'??')
+        >>> print(d | formatlists('{nodename}/{0} : {1} {4}','\n',ctx,'??'))
         ??/Dimm1 : 1 ??
         ??/Dimm2 : 512 ??
     """
@@ -627,17 +629,17 @@ class formatdicts(TextOp):
 
         >>> input = [{'name':'Eric','age':47,'level':'guru'},
         ... {'name':'Guido','age':59,'level':'god'}]
-        >>> print input | formatdicts('{name}({age}) : {level}\n')   #doctest: +NORMALIZE_WHITESPACE
+        >>> print(input | formatdicts('{name}({age}) : {level}\n')   )#doctest: +NORMALIZE_WHITESPACE
         Eric(47) : guru
         Guido(59) : god
-        >>> print input | formatdicts('{name}', ', ')
+        >>> print(input | formatdicts('{name}', ', '))
         Eric, Guido
         >>> ctx = {'today':'2015-12-15'}
-        >>> print input | formatdicts('[{today}] {name}({age}) : {level}','\n',context=ctx)
+        >>> print(input | formatdicts('[{today}] {name}({age}) : {level}','\n',context=ctx))
         [2015-12-15] Eric(47) : guru
         [2015-12-15] Guido(59) : god
         >>> del input[0]['name']
-        >>> print input | formatdicts('[{today}] {name}({age}) : {level}','\n',ctx,'Unknown')
+        >>> print(input | formatdicts('[{today}] {name}({age}) : {level}','\n',ctx,'Unknown'))
         [2015-12-15] Unknown(47) : guru
         [2015-12-15] Guido(59) : god
     """
@@ -877,7 +879,7 @@ class head(TextOp):
         >>> 'a\nb\nc' | head(2).tostr()
         'a\nb'
         >>> for l in 'a\nb\nc' | head(2):
-        ...   print l
+        ...   print(l)
         a
         b
         >>> ['a','b','c'] | head(2).tolist()
@@ -887,7 +889,7 @@ class head(TextOp):
         >>> [('a',1),('b',2),('c',3)] | head(2).tolist()
         [('a', 1), ('b', 2)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | head(2).tolist()
-        [{'val': 1, 'key': 'a'}, {'val': 2, 'key': 'b'}]
+        [{'key': 'a', 'val': 1}, {'key': 'b', 'val': 2}]
     """
     @classmethod
     def op(cls,text,lines,*args,**kwargs):
@@ -911,7 +913,7 @@ class skip(TextOp):
         >>> 'a\nb\nc' | skip(1).tostr()
         'b\nc'
         >>> for l in 'a\nb\nc' | skip(1):
-        ...   print l
+        ...   print(l)
         b
         c
         >>> ['a','b','c'] | skip(1).tolist()
@@ -921,7 +923,7 @@ class skip(TextOp):
         >>> [('a',1),('b',2),('c',3)] | skip(1).tolist()
         [('b', 2), ('c', 3)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | skip(1).tolist()
-        [{'val': 2, 'key': 'b'}, {'val': 3, 'key': 'c'}]
+        [{'key': 'b', 'val': 2}, {'key': 'c', 'val': 3}]
     """
     @classmethod
     def op(cls,text,lines,*args,**kwargs):
@@ -942,7 +944,7 @@ class tail(TextOp):
         >>> 'a\nb\nc' | tail(2).tostr()
         'b\nc'
         >>> for l in 'a\nb\nc' | tail(2):
-        ...   print l
+        ...   print(l)
         b
         c
         >>> ['a','b','c'] | tail(2).tolist()
@@ -952,7 +954,7 @@ class tail(TextOp):
         >>> [('a',1),('b',2),('c',3)] | tail(2).tolist()
         [('b', 2), ('c', 3)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | tail(2).tolist()
-        [{'val': 2, 'key': 'b'}, {'val': 3, 'key': 'c'}]
+        [{'key': 'b', 'val': 2}, {'key': 'c', 'val': 3}]
     """
     @classmethod
     def op(cls,text,lines,*args,**kwargs):
@@ -977,7 +979,7 @@ class less(TextOp):
         >>> 'a\nb\nc' | less(1).tostr()
         'a\nb'
         >>> for l in 'a\nb\nc' | less(1):
-        ...   print l
+        ...   print(l)
         a
         b
         >>> ['a','b','c'] | less(1).tolist()
@@ -987,7 +989,7 @@ class less(TextOp):
         >>> [('a',1),('b',2),('c',3)] | less(1).tolist()
         [('a', 1), ('b', 2)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | less(1).tolist()
-        [{'val': 1, 'key': 'a'}, {'val': 2, 'key': 'b'}]
+        [{'key': 'a', 'val': 1}, {'key': 'b', 'val': 2}]
     """
     @classmethod
     def op(cls,text,lines,*args,**kwargs):
@@ -1013,7 +1015,7 @@ class skess(TextOp):
         >>> 'a\nb\nc' | skess(1,1).tostr()
         'b'
         >>> for l in 'a\nb\nc' | skess(1,1):
-        ...   print l
+        ...   print(l)
         b
         >>> ['a','b','c'] | skess(1,1).tolist()
         ['b']
@@ -1022,7 +1024,7 @@ class skess(TextOp):
         >>> [('a',1),('b',2),('c',3)] | skess(1,1).tolist()
         [('b', 2)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | skess(1,1).tolist()
-        [{'val': 2, 'key': 'b'}]
+        [{'key': 'b', 'val': 2}]
     """
     @classmethod
     def op(cls,text,begin, end, *args,**kwargs):
@@ -1081,7 +1083,7 @@ class findhighlight(TextOp):
         ... the
         ... whole
         ... text'''
-        >>> print s | findhighlight('error',line_nbr=True,ignorecase=True).tostr() # doctest: +NORMALIZE_WHITESPACE
+        >>> print(s | findhighlight('error',line_nbr=True,ignorecase=True).tostr() )# doctest: +NORMALIZE_WHITESPACE
            1
            2    this is
            3    a big
@@ -1094,7 +1096,7 @@ class findhighlight(TextOp):
           10    the
           11    whole
           12    text
-        >>> print s | findhighlight('error',line_nbr=True,ignorecase=True,nlines=1).tostr() # doctest: +NORMALIZE_WHITESPACE
+        >>> print(s | findhighlight('error',line_nbr=True,ignorecase=True,nlines=1).tostr() )# doctest: +NORMALIZE_WHITESPACE
            4    listing
            5 -> with some >>>ERROR<<<
            6    and I want to know
@@ -1110,7 +1112,7 @@ class findhighlight(TextOp):
         buffer = []
         blines = blines or nlines
         elines = elines or nlines
-        if isinstance(pattern,basestring):
+        if isinstance(pattern,str):
             pattern = re.compile(pattern,re.I if ignorecase else 0)
         elines_cptr = 0
         last_found_line = 0
@@ -1205,14 +1207,14 @@ class sed(TextOp):
         for line in cls._tolist(text):
             for pat,repl in zip(pats,repls):
                 if repl is not None:
-                    if isinstance(pat, basestring):
+                    if isinstance(pat, str):
                         pat = re.compile(pat,cls.flags)
-                    if isinstance(line, basestring):
+                    if isinstance(line, str):
                         line = pat.sub(repl,line)
                     elif isinstance(line, list):
                         line = [ pat.sub(repl,stru(item)) for item in line ]
                     elif isinstance(line, dict):
-                        line = dict([(k,pat.sub(repl,stru(v))) for k,v in line.items()])
+                        line = dict([(k,pat.sub(repl,stru(v))) for k,v in list(line.items())])
                     else:
                         line = pat.sub(repl,stru(line))
             yield line
@@ -1253,12 +1255,12 @@ class dostrip(TextOp):
     @classmethod
     def op(cls,text,*args,**kwargs):
         for line in cls._tolist(text):
-            if isinstance(line, basestring):
+            if isinstance(line, str):
                 yield line.strip()
             elif isinstance(line, list):
                 yield [ stru(item).strip() for item in line ]
             elif isinstance(line, dict):
-                yield dict([(k,stru(v).strip()) for k,v in line.items()])
+                yield dict([(k,stru(v).strip()) for k,v in list(line.items())])
             else:
                 yield stru(line).strip()
 
@@ -1323,9 +1325,9 @@ class between(TextOp):
         ... Chaper 3
         ... --------
         ... some other infos'''
-        >>> print s | between('---',r'^\s*$').tostr()
+        >>> print(s | between('---',r'^\s*$').tostr())
         some infos
-        >>> print s | between(['Chapter 2','---'],r'^\s*$').tostr()
+        >>> print(s | between(['Chapter 2','---'],r'^\s*$').tostr())
         infos I want
     """
     flags = 0
@@ -1334,11 +1336,11 @@ class between(TextOp):
     def _build_regex_list(cls,string_or_list):
         if string_or_list is None:
             return []
-        elif isinstance(string_or_list, basestring):
+        elif isinstance(string_or_list, str):
             lst = [string_or_list]
         else:
             lst = list(string_or_list)
-        return [ re.compile(pat,cls.flags) if isinstance(pat, basestring) else pat for pat in lst ]
+        return [ re.compile(pat,cls.flags) if isinstance(pat, str) else pat for pat in lst ]
 
     @classmethod
     def op(cls, text, begin, end, get_begin=None, get_end=None, key=None, *args,**kwargs):
@@ -1460,7 +1462,7 @@ class linetester(TextOp):
     @classmethod
     def castfn(cls, *args, **kwargs):
         first_param = args[0]
-        if isinstance(first_param,basestring):
+        if isinstance(first_param,str):
             return lambda l:cls.make_string(l)
         elif isinstance(first_param,int):
             return lambda l:cls.make_int(l)
@@ -1478,12 +1480,12 @@ class linetester(TextOp):
         castfn = cls.castfn(*args,**kwargs)
 
         if key is not None:
-            if callable(key):
+            if isinstance(key, collections.Callable):
                 getkey = lambda l:key(StrExt(l))
             else:
                 getkey = lambda l:l[key]
         elif attr is not None:
-            if callable(attr):
+            if isinstance(attr, collections.Callable):
                 getkey = lambda l: attr(StrExt(l))
             else:
                 getkey = lambda l: getattr(l,attr)
@@ -1561,7 +1563,7 @@ class inrange(linetester):
         ... {'data':'ccc','date':'2015-09-14'},
         ... {'data':'ddd','date':'2015-11-05'} ]
         >>> logs | inrange('2015-08-12','2015-11-05',key='date').tolist()
-        [{'date': '2015-08-23', 'data': 'bbbb'}, {'date': '2015-09-14', 'data': 'ccc'}]
+        [{'data': 'bbbb', 'date': '2015-08-23'}, {'data': 'ccc', 'date': '2015-09-14'}]
 
         >>> ints = '1\n2\n01\n02\n11\n12\n22\n20'
         >>> ints | inrange(1,3).tolist()
@@ -1661,7 +1663,7 @@ class lessthan(linetester):
         ... {'data':'ccc','date':'2015-09-14'},
         ... {'data':'ddd','date':'2015-11-05'} ]
         >>> logs | lessthan('2015-09-14',key='date').tolist()
-        [{'date': '2015-08-11', 'data': 'aaaa'}, {'date': '2015-08-23', 'data': 'bbbb'}]
+        [{'data': 'aaaa', 'date': '2015-08-11'}, {'data': 'bbbb', 'date': '2015-08-23'}]
         >>> ints = '1\n2\n01\n02\n11\n12\n22\n20'
         >>> ints | lessthan(3).tolist()
         ['1', '2', '01', '02']
@@ -1802,6 +1804,26 @@ class before(between):
     def op(cls, text, pattern, get_end=False, key=None,*args,**kwargs):
         return super(before,cls).op(text,None,pattern,get_end=get_end,key=key)
 
+class until(between):
+    r"""Extract lines before a patterns (case insensitive)
+
+    Works like :class:`textops.before` except that get_end=True (includes first found pattern)
+
+    Args:
+        pattern(str or regex or list): no more lines are yield after reaching this pattern(s)
+        key (int or str): test only one column or one key (optional)
+
+    Yields:
+        str or list or dict: lines before the specified pattern
+
+    Examples:
+        >>> ['a','b','c','d','e','f'] | until('c').tolist()
+        ['a', 'b', 'c']
+    """
+    @classmethod
+    def op(cls, text, pattern, key=None,*args,**kwargs):
+        return super(until,cls).op(text,None,pattern,get_end=True,key=key)
+
 class beforei(before):
     r"""Extract lines before a patterns (case insensitive)
 
@@ -1853,8 +1875,28 @@ class after(between):
         [{'k': 4}, {'k': 5}, {'k': 6}]
     """
     @classmethod
-    def op(cls, text, pattern, get_begin=False,*args,**kwargs):
-        return super(after,cls).op(text,pattern,None,get_begin=get_begin)
+    def op(cls, text, pattern, get_begin=False, key=None, *args,**kwargs):
+        return super(after,cls).op(text,pattern,None,get_begin=get_begin, key=key)
+
+class since(between):
+    r"""Extract lines beginning with and after a patterns
+
+    Works like :class:`textops.after` except that it includes the first parttern found.
+
+    Args:
+        pattern(str or regex or list): start yielding lines after reaching this pattern(s)
+        key (int or str): test only one column or one key (optional)
+
+    Yields:
+        str or list or dict: lines after the specified pattern
+
+    Examples:
+        >>> ['a','b','c','d','e','f'] | since('c').tolist()
+        ['c', 'd', 'e', 'f']
+    """
+    @classmethod
+    def op(cls, text, pattern, key=None, *args,**kwargs):
+        return super(since,cls).op(text,pattern,None,get_begin=True,key=key)
 
 class afteri(after):
     r"""Extract lines after a patterns (case insensitive)
@@ -1991,11 +2033,11 @@ class merge_dicts(TextOp):
         >>> pattern=r'item="(?P<item>[^"]*)" count="(?P<i_count>[^"]*)" price="(?P<i_price>[^"]*)"'
         >>> s='item="col1" count="col2" price="col3"\nitem="col11" count="col22" price="col33"'
         >>> s | cutkv(pattern,key_name='item')                                                      # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-        [{'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}},...
-        {'col11': {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'}}]
+        [{'col1': {'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'}},
+        {'col11': {'item': 'col11', 'i_count': 'col22', 'i_price': 'col33'}}]
         >>> s | cutkv(pattern,key_name='item').merge_dicts()                                        # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-        {'col11': {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'},...
-        'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}}
+        {'col1': {'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'},
+        'col11': {'item': 'col11', 'i_count': 'col22', 'i_price': 'col33'}}
     """
     @classmethod
     def op(cls,text,*args,**kwargs):
@@ -2027,7 +2069,7 @@ class span(TextOp):
         >>> s >> cut().span(3,'-')
         [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
         >>> for x,y,z in s | cut().span(3,'-'):
-        ...    print x,y,z
+        ...    print(x,y,z)
         a - -
         b c -
         d e f
@@ -2148,7 +2190,7 @@ class subitems(TextOp):
     """
     @classmethod
     def op(cls, text, ntab, *args,**kwargs):
-        if isinstance(ntab,basestring):
+        if isinstance(ntab,str):
             ntab = [ int(n) for n in ntab.split(',') ]
         for sublist in cls._tolist(text):
             yield [ sublist[n] for n in ntab ]
@@ -2168,7 +2210,7 @@ class uniq(TextOp):
         >>> s >> uniq()
         ['f', 'a', 'b', 'c', 'e']
         >>> for line in s | uniq():
-        ...     print line
+        ...     print(line)
         f
         a
         b
@@ -2290,7 +2332,7 @@ class splitblock(TextOp):
 
     @classmethod
     def op(cls, text, pattern, include_separator=0, skip_first=False, *args,**kwargs):
-        if isinstance(pattern, basestring):
+        if isinstance(pattern, str):
             pattern = re.compile(pattern,cls.flags)
         blk=[]
         for line in cls._tolist(text):
@@ -2405,7 +2447,7 @@ class resplitblock(TextOp):
 
     @classmethod
     def op(cls, text, pattern, include_separator=0, skip_first=False, *args,**kwargs):
-        if isinstance(pattern, basestring):
+        if isinstance(pattern, str):
             pattern = re.compile(pattern,kwargs.get('flags',cls.flags))
         text = cls._tostr(text)
         blks = []
@@ -2462,7 +2504,7 @@ class aggregate(TextOp):
         ... Message 3
         ... Message 4
         ... '''
-        >>> print s | aggregate(r'(?P<key>> )(?P<msg>.*)').tostr()
+        >>> print(s | aggregate(r'(?P<key>> )(?P<msg>.*)').tostr())
         Message 1
         Message 2 : This is a multiple line message :|In this example,|lines to be aggregated begin with '> '|use '(?P<msg>pattern)' in regex to select what should be aggregated|default join string is '|'
         Message 3
@@ -2476,9 +2518,9 @@ class aggregate(TextOp):
         ... 2016-02-16 11:41:33 info C
         ... 2016-02-16 11:44:26 Message 4
         ... '''
-        >>> print s | aggregate(r'(?P<key>\d+-\d+-\d+ \d+:\d+:\d+) (?P<msg>.*)').tostr()
+        >>> print(s | aggregate(r'(?P<key>\d+-\d+-\d+ \d+:\d+:\d+) (?P<msg>.*)').tostr())
         Message 1|Message 2 info 1|info 2|Message 3 info A|info B|info C|Message 4
-        >>> print s | aggregate(r'(?P<key>\d+-\d+-\d+ \d+:\d+:\d+) (?P<msg>.*)',same_key=True).tostr()
+        >>> print(s | aggregate(r'(?P<key>\d+-\d+-\d+ \d+:\d+:\d+) (?P<msg>.*)',same_key=True).tostr())
         2016-02-16 11:39:03 Message 1
         2016-02-16 11:40:10 Message 2 info 1|info 2
         2016-02-16 11:41:33 Message 3 info A|info B|info C
@@ -2488,14 +2530,14 @@ class aggregate(TextOp):
 
     @classmethod
     def op(cls, text, having, same_key=False, join_str='|', *args,**kwargs):
-        if isinstance(having, basestring):
+        if isinstance(having, str):
             having = re.compile(having,kwargs.get('flags',cls.flags))
 
         buffer=[]
         prev_key=None
         for line in cls._tolist(text):
             key = None
-            if callable(having):
+            if isinstance(having, collections.Callable):
                 key,msg = having(buffer,line)
             else:
                 m = having.match(line)
